@@ -20,14 +20,7 @@
 ** in an effort to keep the tclsqlite.c file pure.
 */
 #include "sqlite3.h"
-#if defined(INCLUDE_SQLITE_TCL_H)
-# include "sqlite_tcl.h"
-#else
-# include "tcl.h"
-# ifndef SQLITE_TCLAPI
-#  define SQLITE_TCLAPI
-# endif
-#endif
+#include "tclsqlite.h"
 
 /* Needed for the setrlimit() system call on unix */
 #if defined(unix)
@@ -66,7 +59,6 @@ const char *sqlite3TestInit(Tcl_Interp *interp){
   extern int Sqlitetest6_Init(Tcl_Interp*);
   extern int Sqlitetest8_Init(Tcl_Interp*);
   extern int Sqlitetest9_Init(Tcl_Interp*);
-  extern int Sqlitetestasync_Init(Tcl_Interp*);
   extern int Sqlitetest_autoext_Init(Tcl_Interp*);
   extern int Sqlitetest_blob_Init(Tcl_Interp*);
   extern int Sqlitetest_demovfs_Init(Tcl_Interp *);
@@ -138,7 +130,6 @@ const char *sqlite3TestInit(Tcl_Interp *interp){
   Sqlitetest6_Init(interp);
   Sqlitetest8_Init(interp);
   Sqlitetest9_Init(interp);
-  Sqlitetestasync_Init(interp);
   Sqlitetest_autoext_Init(interp);
   Sqlitetest_blob_Init(interp);
   Sqlitetest_demovfs_Init(interp);
